%
% This script illustrates the use of the functions in the SPCA toolbox.
%
% The 3 basic functions are:
%
%   initialize_SPCA_2.m... Algorithm initialization: computes the first Principal Curve,
%                        through the given origin, and a set of reference parameters used by the other
%                        functions: local basis at the origin, stopping criterion distance,
%                        and number of bins in each dimension.
%
%   SPCA_2.m.............. Transforms a sample to the intrinsic coordinates
%
%   inv_SPCA_2.m.......... Inverts a sample from the intrinsic coordinates to the original space
%
% Other convenient function to play with:
%
%   plot_axes.m ........ Plots Principal Curve data (as formated in SPCA toolbox)
%

clear all

% WARNING! remember adding the toolbox to the matlab path


% DATA (noisy spiral with increasing noise along the way)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load demo_toy_data

dat = dat(:,1:1000);
dat_test = dat_test(:,1:300);


N_data = length(dat(1,:));          % Number of training samples
N_samples = length(dat_test(1,:));  % Number of test samples


NNB = 100;  % aproximated number of bins for the lattice in this example (not relevant)

for criterion = 3        % metric criterion (2 stands for infomax, and 3 stands for error minimization)
    
    %%%%%%%%%%%%%%%%%%%
    % PARAMETERS OF SPCA
    %%%%%%%%%%%%%%%%%%%
    
    criterio = criterion;   % metric criterion (1 stands for euclidean metric, 2 stands for infomax, and 3 stands for error minimization)
    tol = 0.1;              % tolerance in for the inversion error
    tau = 0.5;              % step size in each PC
    Nv = 0.1*N_data;        % k-neighbors
    pto = [-2;-8.5];        % coordinate origin
    q  = 0.05;              % stiffness
    Btotal = 7;             % number of bits to be distributed among dimensions.

    % If zero is given to the following variables, the initialization
    % function looks for a default value for them.
    
    Nbins = 0;    % Number of bins (resolution) in each dimension e.g. Nbins = [N1, N2, ..., Nd]
    DIST = 0;     % Euclidean distance threshold for the "out-of-the-manifold" condition
    Aref = 0;     % Reference matrix with the local PCA directions at the origin

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %%   INITIALIZATION (on training data):
    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    [init_param] = initialize_SPCA_2(dat,pto,Nv,tau,q,criterio,Btotal,Nbins,DIST,Aref);

        %     figure,plot(dat(1,:),dat(2,:),'.'), axis square, axis equal
        %     plotea_eje(init_param.eucl_PC,'k.-')
        %     title(['Principal Curve euclidean metric'])
        % 
        %     figure,plot(dat(1,:),dat(2,:),'.'), axis square, axis equal
        %     plotea_eje(init_param.PC,'k.-')
        %     title(['Principal Curve modified metric'])

    figure,plot(dat(1,:),dat(2,:),'.'), axis square, axis equal
    plot_axes(init_param,2,1)
    title(['Principal Curves (euclidean metric)'])

    figure,plot(dat(1,:),dat(2,:),'.'), axis square, axis equal
    plot_axes(init_param,2,0)
    title(['First Principal Curve (modified metric)'])

    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %%   TRANSFORM OF TEST DATA:
    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % R2          = PCU transform of test data

    TT = 0;
    for nn = 1:N_samples
        tic
           
        [pto_proximo2(:,nn) R2(:,nn) pto_proximo1(:,nn) R1(:,nn)] = SPCA_2(dat,dat_test(:,nn),init_param,tol);

        aux = toc;
        
        TT = TT +aux;
        ['sample: ' num2str(nn) '  sample-time: ' num2str(aux) '  estimated remaining time: '  num2str((N_samples-nn)*TT/nn)]
    end

    figure,plot(R1(1,:),R1(2,:),'.'), axis square, axis equal
    title(['Transformed data without modifying the local metric'])

    figure,plot(R2(1,:),R2(2,:),'.'), axis square, axis equal
    title(['Transformed data with metric-criterion ' num2str(criterio)])

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %%   VECTOR QUANTIZATION:
    %%   CODEBOOK DESIGN (regular lattice in the transformed domain)
    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


    QQQ = round(sqrt(NNB).*init_param.Nbins./sqrt(prod(init_param.Nbins)));
    ii = find(QQQ/2-round(QQQ/2)==0);
    QQQ(ii) = QQQ(ii)+1;

    mm = min(R2(1,:))-0.00001;
    MM = max(R2(1,:))+0.00001;
    aux = linspace(mm,MM,2*QQQ(1)+1);
    fact1_R2 = aux(2:2:end);

    MM = max(abs(R2(2,:)))-0.15*max(abs(R2(2,:)));
    mm = -MM;
    aux = linspace(mm,MM,2*QQQ(2)+1);
    fact2_R2 = aux(2:2:end);

    [X Y] = meshgrid(fact1_R2,fact2_R2);

    clear dom_QR2
    dom_QR2(1,:) = X(:);
    dom_QR2(2,:) = Y(:);
    
    figure,plot(R2(1,:),R2(2,:),'.'), axis square, axis equal
    hold on
    plot(dom_QR2(1,:),dom_QR2(2,:),'r.')
    title(['Lattice in the transformed domain with metric-criterion ' num2str(criterio)])
    
    % CODEBOOK IN THE INPUT SPACE (inverse of the regular lattice)
    % inv_dom_QR2 = non-uniform lattice in the input domain
    
    inv_dom_QR2 = zeros(size(dom_QR2));
    TT = 0;
    for nn=1:size(dom_QR2,2)
        tic
        
        [inv_dom_QR2(:,nn)] = inv_SPCA_2(dat,dom_QR2(:,nn),init_param);
        
        aux = toc;
          
        TT = TT +aux;
        ['sample: ' num2str(nn) '  sample-time: ' num2str(aux) '  estimated remaining time: '  num2str((size(dom_QR2,2)-nn)*TT/nn)]
    end
    
    figure
    plot(dat(1,:),dat(2,:),'.'), axis square, axis equal
    hold on
    plot(inv_dom_QR2(1,:),inv_dom_QR2(2,:),'r.')
    title(['Lattice in the original domain with metric-criterion ' num2str(criterio)])

    % QUANTIZATION OF TRAINING DATA AND TEST DATA

    for nnn = 1:N_data
        aux = sum((inv_dom_QR2-repmat(dat(:,nnn),1,size(inv_dom_QR2,2))).^2);
        [mm in] = min(aux);
        dat_QR2(:,nnn) = inv_dom_QR2(:,in);
    end

    for nnn = 1:N_samples
        aux = sum((inv_dom_QR2-repmat(dat_test(:,nnn),1,size(inv_dom_QR2,2))).^2);
        [mm in] = min(aux);
        dat_QR2_test(:,nnn) = inv_dom_QR2(:,in);
    end
    
    % QUANTIZATION ERROR
    
    RMSE_R2(criterion) = sqrt(sum(sum((dat - dat_QR2).^2))/prod(size(dat)));
    
    
end

