function [init_param] = initialize_SPCA_2(dat,pto,Nv,tau,q,criterio,Btotal,Nbins,DIST,Aref)

%
% INITIALIZE_SPCA_2 initializes the SPCA algorithm for the manifold described by
% the samples in the variable "dat".
%
% INITIALIZE_SPCA_2 computes the first principal curve though the given origin
% (the vector "origin"), and a set of reference variables useful for the
% application of SPCA (SPCA_2.m and inv_SPCA_2.m).
%
% Inputs (necessary):
%
%  * dat:       Training data, D*N matrix "data" (D is the dimensionality and N is the number of samples)
%  * pto:       Origin in the input space, column vector "origin"
%  * Nv:        Number of neighbors to set the localities, "k"
%  * tau:       Euclidean length of Principal Curve segments "tau"
%  * q:         Stiffness. Weight of the local mean in the modification of
%               local principal direction
%  * criterio:  Criterion for the metric:
%               criterion = 1 Euclidean metric  (gamma = 0)
%               criterion = 2 Infomax metric    (gamma = 1)
%               criterion = 3 Error minimization metric (gamma = 1/3)
%  * Btotal:    number of bits to be distributed among dimensions.
%
% Inputs (optional, to be computed set = 0):
%  * Nbins:     Number of binsper dimension
%  * DIST:      out distance, to stop when computing the PC
%  * Aref       Reference matrix, to sort the PC
%
% Outputs:
%
%  * init_params:   Initialized parameters for using 'SPCA_2.m' or 'inv_SPCA_2.m'
%
% [init_param] =  initialize_SPCA_2(dat,pto,Nv,tau,q,criterio,Btotal,Nbins,DIST,Aref)
%


[DIM Ns] = size(dat);

% DIST

if DIST == 0

    ii_aux = 1:Ns;
    for nn = 1:Ns
        ii = ii_aux(find(ii_aux~=nn));
        N = length(ii);
        aux = sort(sqrt(sum((dat(:,ii)-repmat(dat(:,nn),1,N)).^2)));
        ddd(nn) = min(aux(find(aux~=0)));
    end

    [dist_fuera iii] = max(ddd);

elseif DIST<0

    intervalo=max(dat')-min(dat');
    dist_fuera=sqrt(sum(intervalo.^2))/100;
else
    dist_fuera = DIST;
end

Aref_aux = Aref;

% AREF

if Aref == 0
    dat_aux = dat-repmat(pto,1,Ns);

    C3 = cov2(dat_aux');
    [V3,D3] = eig(C3);

    [Y,iii] = sort(diag(D3),1,'descend');

    Aref_aux =[];
    for nnn = 1:DIM
        Aref_aux(:,nnn) = V3(:,iii(nnn));
    end

    for nnn = 1:DIM
        [MMM,pis] = max(abs(Aref_aux(:,nnn)));
        s=sign(Aref_aux(pis,nnn));
        Aref_aux(:,nnn) = s*Aref_aux(:,nnn);
    end
end

% SPCA axis in the origin

pto_proximo = zeros(size(dat));
resp = zeros(1,size(dat,2));

for direc = 1:DIM
    eje = lanza_axis(dat,pto,Aref_aux,direc,dat,Nv,tau,q,dist_fuera);

    for n=1:size(dat,2)
        [pto_proximo(:,n),resp(n)] = punto_en_eje_NLPCA(dat(:,n),eje);
    end

    d = (sqrt(sum((pto_proximo-dat).^2)));

    [ss ii] = sort(d);
    in = ii(1:round(0.3*size(dat,2)));

    datos(direc,:) = resp(in);
    varianzas(direc) = var(resp(in));

    [p R] = hist(resp(in),sqrt(length(in)));
    delta = R(3)-R(2);
    h(direc) = entropy_mm(p)+log2(delta);

    eje_aux(direc).eje = eje;
end

% SORTING

[Y,iii] = sort(h',1,'descend');

aux_datos = datos;
aux_varianzas = varianzas;
if Aref == 0
    Aref = [];
    for nnn = 1:DIM
        Aref(:,nnn) = Aref_aux(:,iii(nnn));
        hh(nnn) =  h(iii(nnn));
        datos(nnn,:) = aux_datos(iii(nnn),:);
        varianzas(nnn) = aux_varianzas(iii(nnn));
    end
end

% NBINS

if Nbins==0

    RHO2 = prod(varianzas.^(1/DIM));

    % Histograms
    amplitudes = zeros(DIM,round(sqrt(size(datos,2))));
    amplitudes_gorro = amplitudes;
    PDFs = zeros(DIM,round(sqrt(size(datos,2))));
    PDFs_gorro = PDFs;

    for i=1:DIM
        [N,X] = hist(datos(i,:),round(sqrt(size(datos,2))));

        PDFs(i,:) = N/(sum(N)*(X(3)-X(2)));

        amplitudes(i,:) = X;

        amplitudes_gorro(i,:) = X/sqrt(varianzas(i));
        delta_g = amplitudes_gorro(i,3)-amplitudes_gorro(i,2);
        PDFs_gorro(i,:) = N/(sum(N)*delta_g);
        D(i) = max(datos(i,:))-min(datos(i,:));
    end

%     if criterio == 1
        hh2 = D.^2./varianzas;
        HH =  prod(hh2.^(1/DIM));
%     elseif criterio ==2
%         PDFs_gorro = PDFs_gorro(find(PDFs_gorro>0));
%         PDFs_gorro_1 = PDFs_gorro.^(-1);
%         hh2 = sum(PDFs_gorro_1'*delta_g);
%         HH = prod(hh2.^(1/DIM));
%     elseif criterio == 3
%         PDFs_gorro_1_3 = PDFs_gorro.^(1/3);
%         hh2 = sum(PDFs_gorro_1_3'*delta_g).^3;
%         HH = prod(hh2.^(1/DIM));
%     end

    Bi = Btotal/DIM + 0.5*log2(varianzas/RHO2) + 0.5*log2(hh2/HH);
    Bi = Bi.*(Bi>=0);

    Nbins_total = (2.^Btotal);
    Nbins = 2.^Bi;
    Nbins = round(Nbins_total*Nbins./sum(Nbins));
end

% PRINCIPAL EUCL PC

eje_principal = lanza_axis(dat,pto,Aref,1,dat,Nv,tau,q,dist_fuera);

if exist('Aref')
    for kk=1:length(iii)
        ejes_sin_mod(kk).eje = eje_aux(kk).eje;
    end
else
    for kk=1:length(iii)
        ejes_sin_mod(kk).eje = eje_aux(iii(kk)).eje;
    end
end

% PRINCIPAL PC

pto_proximo_principal = zeros(size(dat));
resp_principal = zeros(1,size(dat,2));

for n=1:size(dat,2)
    [pto_proximo_principal(:,n),resp_principal(:,n)] = punto_en_eje_NLPCA(dat(:,n),eje_principal);
end

eje_CRIT = modifica_axis(dat,eje_principal,pto,dat,Nbins(1),Aref,1,Nv,criterio);

pto_proximo_principal_CRIT = zeros(size(dat));
resp_principal_CRIT = zeros(1,size(dat,2));
for n=1:size(dat,2)
    [pto_proximo_principal_CRIT(:,n),resp_principal_CRIT(:,n)] = punto_en_eje_NLPCA(dat(:,n),eje_CRIT);
end

% INITIALIZED PARAMETERS

init_param.Aref = Aref;
init_param.dist_out = dist_fuera;
init_param.Nbins = Nbins;
init_param.eucl_PC = eje_principal;
init_param.points_on_eucl_PC = pto_proximo_principal;
init_param.first_eucl_responses = resp_principal;
init_param.PC = eje_CRIT;
init_param.points_on_PC = pto_proximo_principal_CRIT;
init_param.first_responses = resp_principal_CRIT;
init_param.eucl_PCs = ejes_sin_mod;
init_param.CRIT = criterio;
init_param.origin = pto;
init_param.k = Nv;
init_param.q = q;
init_param.tau = tau;

